@@ -313,17 +313,16 @@ With double prefix argument, specify more connection details."
     (:connected
      (let ((connection (cadr event))
 	   (registerp (plist-get state-data :registerp)))
+     
+       ;; TLS connections leave data in the process buffer, which
+       ;; the XML parser will choke on.
+       (with-current-buffer (process-buffer connection)
+	 (erase-buffer))
 
        (setq state-data (plist-put state-data :connection connection))
 
-       (when (processp connection)
-	 ;; TLS connections leave data in the process buffer, which
-	 ;; the XML parser will choke on.
-	 (with-current-buffer (process-buffer connection)
-	   (erase-buffer))
-
-	 (set-process-filter connection (fsm-make-filter fsm))
-	 (set-process-sentinel connection (fsm-make-sentinel fsm)))
+       (set-process-filter connection (fsm-make-filter fsm))
+       (set-process-sentinel connection (fsm-make-sentinel fsm))
 
        (list :connected state-data)))
 
@@ -991,8 +990,8 @@ Return an fsm result list if it is."
 		   "")
 		 ">
 ")))
-    (jabber-log-xml jc "sending" stream-header)
-    (jabber-send-string jc stream-header)))
+    (jabber-send-string jc stream-header)
+    (jabber-log-xml jc "sending" stream-header)))
 
 (defun jabber-send-string (jc string)
   "Send STRING to the connection JC."
