@@ -1,7 +1,7 @@
 ;; jabber-roster.el - displaying the roster    -*- coding: utf-8; -*-
 
+;; Copyright (C) 2003, 2004, 2007 - Magnus Henoch - mange@freemail.hu
 ;; Copyright (C) 2002, 2003, 2004 - tom berger - object@intelectronica.net
-;; Copyright (C) 2003, 2004 - Magnus Henoch - mange@freemail.hu
 
 ;; This file is a part of jabber.el.
 
@@ -28,7 +28,7 @@
 (defgroup jabber-roster nil "roster display options"
   :group 'jabber)
 
-(defcustom jabber-roster-line-format " %a %c %-25n %u %-8s  %S\n"
+(defcustom jabber-roster-line-format " %a %c %-25n %u %-8s  %S"
   "The format specification of the lines in the roster display.
 
 These fields are available:
@@ -71,7 +71,7 @@ display them: ← → ⇄ ↔"
 	       (cons :format "%v" (const :format "" "both") (string :tag "Both")))
   :group 'jabber-roster)
 
-(defcustom jabber-resource-line-format "     %r - %s (%S), priority %p\n"
+(defcustom jabber-resource-line-format "     %r - %s (%S), priority %p"
   "The format specification of resource lines in the roster display.
 These are displayed when `jabber-show-resources' permits it.
 
@@ -113,6 +113,11 @@ Offline is represented as nil."
 		(const :tag "Always" always))
   :group 'jabber-roster)
 
+(defcustom jabber-show-offline-contacts t
+  "Show offline contacts in roster when non-nil"
+  :type 'boolean
+  :group 'jabber-roster)
+
 (defcustom jabber-remove-newlines t
   "Remove newlines in status messages?
 Newlines in status messages mess up the roster display.  However,
@@ -191,6 +196,8 @@ Trailing newlines are always removed, regardless of this variable."
     (define-key map "a" 'jabber-send-presence)
     (define-key map "g" 'jabber-display-roster)
     (define-key map "S" 'jabber-ft-send)
+    (define-key map "o" 'jabber-roster-toggle-offline-display)
+    (define-key map "H" 'jabber-roster-toggle-binding-display)
     ;;(define-key map "D" 'jabber-disconnect)
     map))
 
@@ -210,15 +217,22 @@ bring up menus of actions.
 
 (put 'jabber-roster-mode 'mode-class 'special)
 
-(defun jabber-switch-to-roster-buffer ()
-  "Switch to roster buffer."
+(defun jabber-switch-to-roster-buffer (&optional jc)
+  "Switch to roster buffer.
+Optional JC argument is ignored; it's there so this function can
+be used in `jabber-post-connection-hooks'."
   (interactive)
-  (switch-to-buffer jabber-roster-buffer))
+  (if (not (get-buffer jabber-roster-buffer))
+      (jabber-display-roster)
+    (switch-to-buffer jabber-roster-buffer)))
 
-(defun jabber-sort-roster ()
+(defun jabber-sort-roster (jc)
   "sort roster according to online status"
-  (setq *jabber-roster*
-	(sort *jabber-roster* #'jabber-roster-sort-items)))
+  (let ((state-data (fsm-get-state-data jc)))
+    (plist-put state-data :roster
+	       (sort
+		(plist-get state-data :roster)
+		#'jabber-roster-sort-items))))
 
 (defun jabber-roster-sort-items (a b)
   "Sort roster items A and B according to `jabber-roster-sort-functions'.
@@ -255,6 +269,16 @@ See `jabber-sort-order' for order used."
      ((string= a-name b-name) 0)
      (t 1))))
 
+(defun jabber-roster-sort-by-group (a b)
+  "Sort roster items by group membership."
+  (flet ((first-group (item) (or (car (get item 'groups)) "")))
+    (let ((a-group (first-group a))
+	  (b-group (first-group b)))
+      (cond
+       ((string-lessp a-group b-group) -1)
+       ((string= a-group b-group) 0)
+       (t 1)))))
+
 (defun jabber-fix-status (status)
   "Make status strings more readable"
   (when status
@@ -265,10 +289,30 @@ See `jabber-sort-order' for order used."
 	(setq status (replace-match " " t t status))))
     status))
 
-(defvar jabber-roster-positions nil
-  "Alist tracking positions of items in the roster.
-Keys are bare JID symbols.  Values are conses of markers,
-marking the extent of the roster entry.")
+(defvar jabber-roster-ewoc nil
+  "Ewoc displaying the roster.
+There is only one; we don't rely on buffer-local variables or
+such.")
+
+(defun jabber-roster-filter-display (buddies)
+  "Filter BUDDIES for items to be displayed in the roster"
+  (remove-if-not (lambda (buddy) (or jabber-show-offline-contacts
+				     (get buddy 'connected)))
+		 buddies))
+
+(defun jabber-roster-toggle-offline-display ()
+  "Toggle display of offline contacts."
+  (interactive)
+  (setq jabber-show-offline-contacts
+	(not jabber-show-offline-contacts))
+  (jabber-display-roster))
+
+(defun jabber-roster-toggle-binding-display ()
+  "Toggle display of the roster binding text."
+  (interactive)
+  (setq jabber-roster-show-bindings
+	(not jabber-roster-show-bindings))
+  (jabber-display-roster))
 
 (defun jabber-display-roster ()
   "switch to the main jabber buffer and refresh the roster display to reflect the current information"
@@ -283,51 +327,63 @@ marking the extent of the roster entry.")
     (let ((current-line (and (fboundp 'line-number-at-pos) (line-number-at-pos)))
 	  (current-column (current-column)))
       (erase-buffer)
-      (setq jabber-roster-positions nil)
-      (insert (jabber-propertize jabber-server 'face 'jabber-title-large) "\n")
+      (setq jabber-roster-ewoc nil)
+      (insert (jabber-propertize "Jabber roster" 'face 'jabber-title-large) "\n")
       (when jabber-roster-show-bindings
 	(insert "RET      Open chat buffer        C-k      Delete roster item
 e        Edit item               s        Send subscription request
 q        Bury buffer             i        Get disco items
 I        Get disco info          b        Browse
 j        Join groupchat (MUC)    v        Get client version
-a        Send presence
+a        Send presence           o        Show offline contacts on/off
 C-c C-c  Chat menu               C-c C-m  Multi-User Chat menu
 C-c C-i  Info menu               C-c C-r  Roster menu
 C-c C-s  Service menu
+
+H        Toggle displaying this text
 "))
       (insert "__________________________________\n\n")
-      (let ((map (make-sparse-keymap)))
-	(define-key map [mouse-2] #'jabber-send-presence)
-	(insert (jabber-propertize (concat (format " - %s"
-						   (cdr (assoc *jabber-current-show* jabber-presence-strings)))
-					   (if (not (zerop (length *jabber-current-status*)))
-					       (format " (%s)"
-						       (jabber-fix-status *jabber-current-status*)))
-					   " -")
-				   'face (or (cdr (assoc *jabber-current-show* jabber-presence-faces))
-					     'jabber-roster-user-online)
-				   ;;'mouse-face (cons 'background-color "light grey")
-				   'keymap map)
-		"\n__________________________________\n\n"))
+      (if (null jabber-connections)
+	  (insert "Not connected\n")
+	(let ((map (make-sparse-keymap)))
+	  (define-key map [mouse-2] #'jabber-send-presence)
+	  (insert (jabber-propertize (concat (format " - %s"
+						     (cdr (assoc *jabber-current-show* jabber-presence-strings)))
+					     (if (not (zerop (length *jabber-current-status*)))
+						 (format " (%s)"
+							 (jabber-fix-status *jabber-current-status*)))
+					     " -")
+				     'face (or (cdr (assoc *jabber-current-show* jabber-presence-faces))
+					       'jabber-roster-user-online)
+				     ;;'mouse-face (cons 'background-color "light grey")
+				     'keymap map)
+		  "\n")))
 
-      (jabber-sort-roster)
-      (dolist (buddy *jabber-roster*)
-	(let ((entry-start (point)))
-	  (jabber-display-roster-entry buddy)
-
-	  ;; Keep track of this roster entry's position
-	  (let ((entry (assq buddy jabber-roster-positions)))
-	    (unless entry
-	      (setq entry (cons buddy nil))
-	      (push entry jabber-roster-positions))
-	      (let ((marker-start (set-marker (make-marker) entry-start))
-		    (marker-end (set-marker (make-marker) (point))))
-		;; Text is inserted before start markers, but after
-		;; end markers.
-		(set-marker-insertion-type marker-start t)
-		(setcdr entry (cons marker-start marker-end))))))
-      (insert "__________________________________")
+      (dolist (jc jabber-connections)
+	;; We sort everything before putting it in the ewoc
+	(jabber-sort-roster jc)
+	(let ((before-ewoc (point))
+	      (ewoc (ewoc-create 
+		     (lexical-let ((jc jc))
+		       (lambda (buddy)
+			 (jabber-display-roster-entry jc buddy)))
+		     (concat
+		      (jabber-propertize (concat
+					  (plist-get (fsm-get-state-data jc) :username)
+					  "@"
+					  (plist-get (fsm-get-state-data jc) :server))
+					 'face 'jabber-title-medium)
+		      "\n__________________________________\n")
+		     "__________________________________")))
+	  (plist-put (fsm-get-state-data jc) :roster-ewoc ewoc)
+	  (dolist (buddy (jabber-roster-filter-display
+			  (plist-get (fsm-get-state-data jc) :roster)))
+	    (ewoc-enter-last ewoc buddy))
+	  (goto-char (point-max))
+	  (insert "\n")
+	  (put-text-property before-ewoc (point)
+			     'jabber-account jc)))
+      
       (goto-char (point-min))
       (setq buffer-read-only t)
       (if (interactive-p)
@@ -337,7 +393,7 @@ C-c C-s  Service menu
 	(goto-line current-line)
 	(move-to-column current-column)))))
 
-(defun jabber-display-roster-entry (buddy)
+(defun jabber-display-roster-entry (jc buddy)
   "Format and insert a roster entry for BUDDY at point.
 BUDDY is a JID symbol."
   (let ((buddy-str (format-spec jabber-roster-line-format
@@ -369,7 +425,9 @@ BUDDY is a JID symbol."
 			  'help-echo
 			  (symbol-name buddy)
 			  'jabber-jid
-			  (symbol-name buddy))
+			  (symbol-name buddy)
+			  'jabber-account
+			  jc)
 			 buddy-str)
     ;; (let ((map (make-sparse-keymap))
     ;; 	      (chat-with-func (make-symbol (concat "jabber-chat-with" (symbol-name buddy)))))
@@ -411,45 +469,61 @@ BUDDY is a JID symbol."
 				  (or (cdr (assoc (plist-get (cdr resource) 'show) jabber-presence-faces))
 				      'jabber-roster-user-online)
 				  'jabber-jid
-				  (format "%s/%s" (symbol-name buddy) (car resource)))
+				  (format "%s/%s" (symbol-name buddy) (car resource))
+				  'jabber-account
+				  jc)
 				 resource-str)
-	    (insert resource-str)))))))
+	    (insert "\n" resource-str)))))))
 
-(defun jabber-presence-update-roster (who &rest ignore)
-  "Update roster without redrawing all of it, if possible.
-WHO is a JID symbol."
-  
-  (let* ((bare-jid (jabber-jid-symbol 
-		    (jabber-jid-user
-		     (symbol-name who))))
-	 (entry (assq bare-jid jabber-roster-positions))
-	 (inhibit-read-only t))
-    (jabber-sort-roster)
-    (if (null entry)
+(defun jabber-roster-update (jc new-items changed-items deleted-items)
+  "Update roster, in memory and on display.
+Add NEW-ITEMS, update CHANGED-ITEMS and remove DELETED-ITEMS, all
+three being lists of JID symbols."
+  (let ((roster (plist-get (fsm-get-state-data jc) :roster))
+	(ewoc (plist-get (fsm-get-state-data jc) :roster-ewoc)))
+    (dolist (delete-this deleted-items)
+      (setq roster (delq delete-this roster)))
+    (setq roster (append new-items roster))
+    (plist-put (fsm-get-state-data jc) :roster roster)
+
+    ;; If there is no ewoc yet, create the roster buffer.
+    (if (null ewoc)
 	(jabber-display-roster)
-      (let ((old-start (cadr entry))
-	    (old-end (cddr entry))
-	    (insert-before-this (cadr (memq bare-jid *jabber-roster*))))
-	(with-current-buffer jabber-roster-buffer
-	  (delete-region old-start old-end)
-	  (save-excursion
-	    (let ((new-start 
-		   (marker-position
-		    (if insert-before-this
-			;; If this is not the last entry, go to start
-			;; position of next entry.
-			(cadr (assq insert-before-this jabber-roster-positions))
-		      ;; If this is the last entry, go to end position of second
-		      ;; to last entry.
-		      (cddr (car (last jabber-roster-positions 2)))))))
-	      (goto-char new-start)
-	      (jabber-display-roster-entry bare-jid)
-	      (let ((marker-start (set-marker (make-marker) new-start))
-		    (marker-end (set-marker (make-marker) (point))))
-		;; Text is inserted before start markers, but after
-		;; end markers.
-		(set-marker-insertion-type marker-start t)
-		(setcdr entry (cons marker-start marker-end))))))))))
+      ;; Otherwise, do incremental changes.
+
+      ;; The changed items need to be resorted, so we start by removing
+      ;; them as well.
+      (ewoc-filter ewoc
+		   (lambda (a) (not (or (member a changed-items)
+					(member a deleted-items)))))
+  
+      ;; Now, insert items into ewoc.
+      (let* ((to-be-inserted
+	      (sort (jabber-roster-filter-display
+		     (append new-items changed-items))
+		    #'jabber-roster-sort-items))
+	     (where (ewoc-nth ewoc 0)))
+	(while to-be-inserted
+	  (cond
+	   ;; If we are at the end of the ewoc, put all elements there.
+	   ((null where)
+	    (dolist (a to-be-inserted)
+	      (ewoc-enter-last ewoc a))
+	    (setq to-be-inserted nil))
+	   ;; If the next element should go here, put it here.
+	   ((jabber-roster-sort-items (car to-be-inserted)
+				      (ewoc-data where))
+	    (ewoc-enter-before ewoc where
+			       (car to-be-inserted))
+	    (setq to-be-inserted (cdr to-be-inserted)))
+	   ;; Else, advance through the ewoc.
+	   (t
+	    (setq where (ewoc-next ewoc where)))))))))
+
+(defalias 'jabber-presence-update-roster 'ignore)
+;;jabber-presence-update-roster is not needed anymore.
+;;Its work is done in `jabber-process-presence'."
+(make-obsolete 'jabber-presence-update-roster 'ignore)
 
 (defun jabber-go-to-next-jid ()
   "Move the cursor to the next jid in the buffer"
